declare const _default: "\n#ifdef USE_CLEAR_COAT\n  uniform float uClearCoat;\n  uniform float uClearCoatRoughness;\n\n  #ifdef USE_CLEAR_COAT_MAP\n    uniform sampler2D uClearCoatMap;\n\n    #ifdef USE_CLEAR_COAT_MAP_TEX_COORD_TRANSFORM\n      uniform mat3 uClearCoatMapTexCoordTransform;\n    #endif\n\n    void getClearCoat(inout PBRData data) {\n      #ifdef USE_CLEAR_COAT_MAP_TEX_COORD_TRANSFORM\n        vec2 texCoord = getTextureCoordinates(data, CLEAR_COAT_MAP_TEX_COORD_INDEX, uClearCoatMapTexCoordTransform);\n      #else\n        vec2 texCoord = getTextureCoordinates(data, CLEAR_COAT_MAP_TEX_COORD_INDEX);\n      #endif\n\n      data.clearCoat = uClearCoat * texture2D(uClearCoatMap, texCoord).r;\n    }\n  #else\n    void getClearCoat(inout PBRData data) {\n      data.clearCoat = uClearCoat;\n    }\n  #endif\n\n  #ifdef USE_CLEAR_COAT_ROUGHNESS_MAP\n    uniform sampler2D uClearCoatRoughnessMap;\n\n    #ifdef USE_CLEAR_COAT_ROUGHNESS_MAP_TEX_COORD_TRANSFORM\n      uniform mat3 uClearCoatRoughnessMapTexCoordTransform;\n    #endif\n\n    void getClearCoatRoughness(inout PBRData data) {\n      #ifdef USE_CLEAR_COAT_ROUGHNESS_MAP_TEX_COORD_TRANSFORM\n        vec2 texCoord = getTextureCoordinates(data, CLEAR_COAT_ROUGHNESS_MAP_TEX_COORD_INDEX, uClearCoatRoughnessMapTexCoordTransform);\n      #else\n        vec2 texCoord = getTextureCoordinates(data, CLEAR_COAT_ROUGHNESS_MAP_TEX_COORD_INDEX);\n      #endif\n\n      data.clearCoatRoughness = uClearCoatRoughness * texture2D(uClearCoatRoughnessMap, texCoord).g;\n    }\n  #else\n    void getClearCoatRoughness(inout PBRData data) {\n      data.clearCoatRoughness = uClearCoatRoughness;\n    }\n  #endif\n\n  #ifdef USE_CLEAR_COAT_NORMAL_MAP\n    uniform sampler2D uClearCoatNormalMap;\n    uniform float uClearCoatNormalMapScale;\n\n    #ifdef USE_CLEAR_COAT_NORMAL_MAP_TEX_COORD_TRANSFORM\n      uniform mat3 uClearCoatNormalMapTexCoordTransform;\n    #endif\n\n    void getClearCoatNormal(inout PBRData data) {\n      #ifdef USE_CLEAR_COAT_NORMAL_MAP_TEX_COORD_TRANSFORM\n        vec2 texCoord = getTextureCoordinates(data, CLEAR_COAT_NORMAL_MAP_TEX_COORD_INDEX, uClearCoatNormalMapTexCoordTransform);\n      #else\n        vec2 texCoord = getTextureCoordinates(data, CLEAR_COAT_NORMAL_MAP_TEX_COORD_INDEX);\n      #endif\n\n      vec3 normalMap = texture2D(uClearCoatNormalMap, texCoord).rgb * 2.0 - 1.0;\n      normalMap.y *= uClearCoatNormalMapScale;\n      normalMap = normalize(normalMap);\n\n      vec3 N = normalize(data.normalView);\n      vec3 V = normalize(data.eyeDirView);\n\n      vec3 normalView;\n\n      #ifdef USE_TANGENTS\n        vec3 bitangent = cross(N, data.tangentView.xyz) * sign(data.tangentView.w);\n        mat3 TBN = mat3(data.tangentView.xyz, bitangent, N);\n        normalView = normalize(TBN * normalMap);\n      #else\n        normalMap.xy *= float(gl_FrontFacing) * 2.0 - 1.0;\n        // make the output normalView match glTF expected right handed orientation\n        normalMap.y *= -1.0;\n        normalView = perturb(normalMap, N, V, texCoord);\n      #endif\n\n      data.clearCoatNormal = normalize(vec3(data.inverseViewMatrix * vec4(normalView, 0.0)));\n    }\n  #else\n    void getClearCoatNormal(inout PBRData data) {\n      // geometric normal without perturbation from normalMap\n      // this normal is in world space\n      data.clearCoatNormal = normalize(vec3(data.inverseViewMatrix * vec4(normalize(vNormalView), 0.0)));\n    }\n  #endif\n\n\n  // IOR = 1.5, F0 = 0.04\n  // as material is no longer in contact with air we calculate new IOR on the\n  // clear coat and material interface\n  vec3 f0ClearCoatToSurface(const vec3 f0) {\n    return saturate(f0 * (f0 * (0.941892 - 0.263008 * f0) + 0.346479) - 0.0285998);\n  }\n\n  float clearCoatBRDF(const PBRData data, const vec3 h, float NoH, float LoH, out float Fcc) {\n    #if defined(USE_NORMAL_MAP) || defined(USE_CLEAR_COAT_NORMAL_MAP)\n      float clearCoatNoH = saturate(dot(data.clearCoatNormal, h));\n    #else\n      float clearCoatNoH = NoH;\n    #endif\n    float D = D_GGX(data.clearCoatLinearRoughness, clearCoatNoH, h, data.normalWorld);\n    float V = V_Kelemen(LoH);\n    // air-polyurethane interface has IOR = 1.5 -> F0 = 0.04\n    float F = F_Schlick(0.04, 1.0, LoH) * data.clearCoat;\n\n    Fcc = F;\n    return D * V * F;\n  }\n#endif\n";
export default _default;
