declare const _default: "\nstruct Light {\n  vec3 l;\n  vec4 color;\n  float attenuation;\n};\n\n// https://seblagarde.files.wordpress.com/2015/07/course_notes_moving_frostbite_to_pbr_v32.pdf\nfloat getDistanceAttenuation(const highp vec3 posToLight, float falloff) {\n  // Square Falloff Attenuation\n  float distanceSquare = dot(posToLight, posToLight);\n  float factor = distanceSquare * falloff;\n  float smoothFactor = saturate(1.0 - factor * factor);\n  float attenuation = smoothFactor * smoothFactor;\n\n  return attenuation * 1.0 / max(distanceSquare, 1e-4);\n}\n\nfloat getAngleAttenuation(const vec3 lightDir, const vec3 l, const vec2 scaleOffset) {\n  float cd = dot(lightDir, l);\n  float attenuation  = saturate(cd * scaleOffset.x + scaleOffset.y);\n  return attenuation * attenuation;\n}\n\nvec2 compensateStretch(vec2 uv) {\n  return uv;\n  // float u = uv.x;\n  // u = (u - 0.5) * 1.1 + 0.5;\n  // return vec2(u, uv.y);\n}\n\n\nvoid getSurfaceShading(inout PBRData data, Light light, float illuminated) {\n  vec3 N = data.normalWorld;\n  vec3 V = data.viewWorld;\n  vec3 L = normalize(light.l);\n  vec3 H = normalize(V + L);\n\n  float NdotV = saturate(abs(dot(N, V)) + FLT_EPS);\n  float NdotL = saturate(dot(N, L));\n\n  #ifndef USE_TRANSMISSION\n  if (NdotL <= 0.0) return;\n  #endif\n\n  float NdotH = saturate(dot(N, H));\n  float LdotH = saturate(dot(L, H));\n  float HdotV = max(0.0, dot(H, V));\n\n  vec3 F = SpecularReflection(data.f0, HdotV);\n\n  float D = MicrofacetDistribution(data.linearRoughness, NdotH);\n  float Vis = VisibilityOcclusion(data.linearRoughness, NdotL, NdotV);\n\n  //TODO: switch to linear colors\n  vec3 lightColor = decode(light.color, 3).rgb;\n\n  vec3 Fd = DiffuseLambert() * data.diffuseColor;\n  vec3 Fr = F * Vis * D;\n  vec3 Fs = vec3(0.0);\n\n  //TODO: energy compensation\n  float energyCompensation = 1.0;\n\n  #ifdef USE_SHEEN\n    Fs = EvaluateSheen(data, NdotH, NdotV, NdotL);\n  #endif\n\n  #ifdef USE_CLEAR_COAT\n    float Fcc;\n    float clearCoat = clearCoatBRDF(data, H, NdotH, LdotH, Fcc);\n    float attenuation = 1.0 - Fcc;\n\n    vec3 color = (Fs + Fd + Fr * (energyCompensation * attenuation)) * attenuation * NdotL;\n\n    // direct light still uses NdotL but clear coat needs separate dot product when using normal map\n    // if only normal map is present not clear coat normal map, we will get smooth coating on top of bumpy surface\n    #if defined(USE_NORMAL_MAP) || defined(USE_CLEAR_COAT_NORMAL_MAP)\n      float clearCoatNoL = saturate(dot(data.clearCoatNormal, light.l));\n      color += clearCoat * clearCoatNoL;\n    #else\n      color += clearCoat * NdotL;\n    #endif\n  #else\n    vec3 color = (Fs + Fd + Fr * energyCompensation) * NdotL;\n  #endif\n\n\n\n  data.directColor += (color * lightColor) * (light.color.a * light.attenuation * illuminated);\n\n  #ifdef USE_TRANSMISSION\n  // data.directDiffuse = texture2D(uCaptureTexture, gl_FragCoord.xy / uScreenSize.xy).rgb;\n\n  float uIOR = 1.0;\n  vec3 IoR_Values = mix(vec3(1.0), vec3(1.14, 1.12, 1.10), uIOR);\n\n  vec3 incident = normalize(data.eyeDirWorld);\n\n  float f = F_Schlick(0.04, 1.0, abs(dot(data.viewWorld, data.normalWorld)));\n\n  vec3 refractColor = vec3(0.0);\n  // #ifdef USE_REFLECTION_PROBES\n  // refractColor.x = texture2D(uEnvMap, envMapEquirect(refract(incident, normalWorld, IoR_Values.x))).x;\n  // refractColor.y = texture2D(uEnvMap, envMapEquirect(refract(incident, normalWorld, IoR_Values.y))).y;\n  // refractColor.z = texture2D(uEnvMap, envMapEquirect(refract(incident, normalWorld, IoR_Values.z))).z;\n  // #endif\n  float refractAmount = uRefraction;\n\n  vec3 reflectColor = vec3(0.0);\n\n  #ifdef USE_REFLECTION_PROBES\n  // reflectColor = texture2D(uReflectionMap, envMapOctahedral(reflect(-data.viewWorld, data.normalWorld), 0.0, 0.0)).rgb;\n  #endif\n\n  // vec3 IoR_Values = mix(vec3(1.0), vec3(1.14, 1.12, 1.10), uIOR);\n  float level = clamp(data.roughness + (1.0 - data.opacity), 0.0, 1.0) * 5.0; //Opacity Hack\n  vec2 uv = gl_FragCoord.xy / uScreenSize.xy;\n  vec2 refractionAspect = vec2(1.0 * uScreenSize.y/uScreenSize.x, 1.0);\n  refractColor.x = texture2DLodEXT(uCaptureTexture, compensateStretch(uv + refractAmount * refract(incident, data.normalWorld, 1.0 / IoR_Values.x).xy), level).x;\n  refractColor.y = texture2DLodEXT(uCaptureTexture, compensateStretch(uv + refractAmount * refract(incident, data.normalWorld, 1.0 / IoR_Values.y).xy), level).y;\n  refractColor.z = texture2DLodEXT(uCaptureTexture, compensateStretch(uv + refractAmount * refract(incident, data.normalWorld, 1.0 / IoR_Values.z).xy), level).z;\n  // refractColor.x = texture2D(uCaptureTexture, compensateStretch(uv + refractionAspect * refractAmount * refract(incident, data.normalWorld, 1.0 / IoR_Values.x).xy)).x;\n  // refractColor.y = texture2D(uCaptureTexture, compensateStretch(uv + refractionAspect * refractAmount * refract(incident, data.normalWorld, 1.0 / IoR_Values.y).xy)).y;\n  // refractColor.z = texture2D(uCaptureTexture, compensateStretch(uv + refractionAspect * refractAmount * refract(incident, data.normalWorld, 1.0 / IoR_Values.z).xy)).z;\n\n  data.directColor *= 0.3;\n  data.directColor += data.diffuseColor * mix(refractColor, reflectColor, f);\n  // data.directColor = vec3(uScreenSize.xy, 0.0);\n  // data.directColor += refractColor;\n  // data.directColor = vec3(f);\n  // data.directColor = reflectColor;\n  // data.directColor = 0.2 + texture2D(uCaptureTexture, uv).rgb;\n  // data.directColor = vec3(uv, 1.0);\n  // data.directColor = vec3(1.0, 0.0, 1.0);\n  // data.directColor = texture2D(uCaptureTexture, uv).xyz;\n  #endif\n}\n";
export default _default;
