declare const _default: "\n#if NUM_SPOT_LIGHTS > 0\n\nstruct SpotLight {\n  vec3 position;\n  vec3 direction;\n  vec4 color;\n  float innerAngle;\n  float angle;\n  float range;\n  mat4 projectionMatrix;\n  mat4 viewMatrix;\n  bool castShadows;\n  float near;\n  float far;\n  float bias;\n  vec2 shadowMapSize;\n};\n\nuniform SpotLight uSpotLights[NUM_SPOT_LIGHTS];\nuniform sampler2D uSpotLightShadowMaps[NUM_SPOT_LIGHTS];\n\nvoid EvaluateSpotLight(inout PBRData data, SpotLight light, sampler2D shadowMap) {\n  vec4 lightViewPosition = light.viewMatrix * vec4(vPositionWorld, 1.0); // TODO: move in the vertex shader\n  float lightDistView = -lightViewPosition.z;\n  vec4 lightDeviceCoordsPosition = light.projectionMatrix * lightViewPosition;\n  vec3 lightDeviceCoordsPositionNormalized = lightDeviceCoordsPosition.xyz / lightDeviceCoordsPosition.w;\n  vec2 lightUV = lightDeviceCoordsPositionNormalized.xy * 0.5 + 0.5;\n\n  float illuminated = bool(light.castShadows) ? getShadow(shadowMap, light.shadowMapSize, lightUV, lightDistView - light.bias, light.near, light.far) : 1.0;\n\n  if (illuminated > 0.0) {\n    vec3 posToLight = light.position - data.positionWorld;\n\n    float invSqrFalloff = 1.0 / pow(light.range, 2.0);\n    float attenuation = getDistanceAttenuation(posToLight, invSqrFalloff);\n\n    // TODO: luminous power to intensity\n    float innerAngle = light.innerAngle;\n    float cosOuter = cos(light.angle);\n    float cosInner = cos(innerAngle);\n    float cosOuterSquared = cosOuter * cosOuter;\n    float scale = 1.0 / max(1.0 / 1024.0, cosInner - cosOuter);\n    float offset = -cosOuter * scale;\n\n    vec2 scaleOffset = vec2(scale, offset);\n    attenuation *= getAngleAttenuation(-light.direction, normalize(posToLight), scaleOffset);\n\n    Light l;\n    l.l = normalize(posToLight);\n    l.color = light.color;\n    l.attenuation = attenuation;\n    getSurfaceShading(data, l, illuminated);\n  }\n}\n#endif\n";
export default _default;
