declare const _default: "\n#ifdef USE_SHEEN\n    uniform vec4 uSheenColor; // TODO: gltf assumes sRGB color, not linear\n    uniform float uSheenRoughness;\n\n    #ifdef USE_SHEEN_COLOR_MAP\n    uniform sampler2D uSheenColorMap; // assumes sRGB color, not linear\n\n    #ifdef USE_SHEEN_COLOR_MAP_TEX_COORD_TRANSFORM\n        uniform mat3 uSheenColorMapTexCoordTransform;\n    #endif\n\n    void getSheenColor(inout PBRData data) {\n        #ifdef USE_SHEEN_COLOR_MAP_TEX_COORD_TRANSFORM\n        vec2 texCoord = getTextureCoordinates(data, SHEEN_COLOR_MAP_TEX_COORD_INDEX, uSheenColorMapTexCoordTransform);\n        #else\n        vec2 texCoord = getTextureCoordinates(data, SHEEN_COLOR_MAP_TEX_COORD_INDEX);\n        #endif\n        vec4 texelColor = texture2D(uSheenColorMap, texCoord);\n\n        #if !defined(DEPTH_PASS_ONLY) && !defined(DEPTH_PRE_PASS_ONLY)\n        data.sheenColor = decode(uSheenColor, 3).rgb * decode(texelColor, 3).rgb;\n        data.sheenRoughness = uSheenRoughness * texelColor.a;\n        #endif\n    }\n    #else\n    void getSheenColor(inout PBRData data) {\n        #if !defined(DEPTH_PASS_ONLY) && !defined(DEPTH_PRE_PASS_ONLY)\n        data.sheenColor = decode(uSheenColor, 3).rgb;\n        #endif\n        data.sheenRoughness = uSheenRoughness;\n    }\n    #endif\n\n    float Sheen_l(float x, float alphaG) {\n        float oneMinusAlphaSq = (1.0 - alphaG) * (1.0 - alphaG);\n        float a = mix(21.5473, 25.3245, oneMinusAlphaSq);\n        float b = mix(3.82987, 3.32435, oneMinusAlphaSq);\n        float c = mix(0.19823, 0.16801, oneMinusAlphaSq);\n        float d = mix(-1.97760, -1.27393, oneMinusAlphaSq);\n        float e = mix(-4.32054, -4.85967, oneMinusAlphaSq);\n        return a / (1.0 + b * pow(x, c)) + d * x + e;\n    }\n\n    float lambdaSheen(float cosTheta, float alphaG) {\n        return abs(cosTheta) < 0.5 ? exp(Sheen_l(cosTheta, alphaG)) : exp(2.0 * Sheen_l(0.5, alphaG) - Sheen_l(1.0 - cosTheta, alphaG));\n    }\n\n    vec3 EvaluateSheen(inout PBRData data, float NdotH, float NdotV, float NdotL) {\n        float alphaG = data.sheenRoughness * data.sheenRoughness;\n        float invR = 1.0 / alphaG;\n\n        float cos2h = NdotH * NdotH;\n        float sin2h = 1.0 - cos2h;\n        float sheenDistribution = (2.0 + invR) * pow(sin2h, invR * 0.5) / (2.0 * PI);\n\n        float sheenVisibility = 1.0 / ((1.0 + lambdaSheen(NdotV, alphaG) + lambdaSheen(NdotL, alphaG)) * (4.0 * NdotV * NdotL));\n        float sheenFresnel = 1.0;\n        vec3 sheen = data.sheenColor * sheenFresnel * sheenDistribution * sheenVisibility;\n        data.sheen += sheen;\n\n        return sheen;\n\n        // TODO: add albedo scaling\n        // https://github.com/KhronosGroup/glTF/blob/master/extensions/2.0/Khronos/KHR_materials_sheen/README.md#albedo-scaling-technique\n    }\n#endif\n";
export default _default;
