declare const _default: "\n#ifdef USE_SPECULAR_GLOSSINESS_WORKFLOW\n  uniform vec4 uDiffuse;\n  uniform vec3 uSpecular;\n  uniform float uGlossiness;\n\n  #ifdef USE_DIFFUSE_MAP\n    uniform sampler2D uDiffuseMap;\n    uniform float uDiffuseMapEncoding;\n\n    #ifdef USE_DIFFUSE_MAP_TEX_COORD_TRANSFORM\n      uniform mat3 uDiffuseMapTexCoordTransform;\n    #endif\n\n    vec4 getDiffuse(in PBRData data) {\n      // assumes sRGB texture\n      #ifdef USE_DIFFUSE_MAP_TEX_COORD_TRANSFORM\n        vec2 texCoord = getTextureCoordinates(data, DIFFUSE_MAP_TEX_COORD_INDEX, uDiffuseMapTexCoordTransform);\n      #else\n        vec2 texCoord = getTextureCoordinates(data, DIFFUSE_MAP_TEX_COORD_INDEX);\n      #endif\n      vec4 texelColor = texture2D(uDiffuseMap, texCoord);\n      return vec4(decode(uDiffuse, 3).rgb, uDiffuse.a) * vec4(decode(texelColor, 3).rgb, texelColor.a);\n    }\n  #else\n    vec4 getDiffuse(in PBRData data) {\n      return vec4(decode(uDiffuse, 3).rgb, uDiffuse.a);\n    }\n  #endif\n\n  #ifdef USE_SPECULAR_GLOSSINESS_MAP\n    uniform sampler2D uSpecularGlossinessMap;\n\n    #ifdef USE_SPECULAR_GLOSSINESS_MAP_TEX_COORD_TRANSFORM\n      uniform mat3 uSpecularGlossinessMapTexCoordTransform;\n    #endif\n\n    vec4 getSpecularGlossiness(in PBRData data) {\n      // assumes specular is sRGB and glossiness is linear\n      #ifdef USE_SPECULAR_GLOSSINESS_MAP_TEX_COORD_TRANSFORM\n        vec2 texCoord = getTextureCoordinates(data, SPECULAR_GLOSSINESS_MAP_TEX_COORD_INDEX, uSpecularGlossinessMapTexCoordTransform);\n      #else\n        vec2 texCoord = getTextureCoordinates(data, SPECULAR_GLOSSINESS_MAP_TEX_COORD_INDEX);\n      #endif\n      vec4 specGloss = texture2D(uSpecularGlossinessMap, texCoord);\n      //TODO: should i move uSpecular to linear?\n      return vec4(uSpecular, uGlossiness) * vec4(decode(vec4(specGloss.rgb, 1.0), 3).rgb, specGloss.a);\n    }\n  #else\n    vec4 getSpecularGlossiness(in PBRData data) {\n      return vec4(uSpecular, uGlossiness);\n    }\n  #endif\n\n  // assumes linear color\n  float perceivedBrightness(vec3 c) {\n    return 0.299 * c.r + 0.587 * c.g + 0.114 * c.b;\n  }\n\n  float maxComponent(vec3 c) {\n    return max(c.r, max(c.b, c.g));\n  }\n\n  float solveMetallic(float diffuse, float specular, float oneMinusSpecularStrength) {\n    if (specular < 0.04) {\n      return 0.0;\n    }\n\n    float a = 0.04;\n    float b = diffuse * oneMinusSpecularStrength / (1.0 - a) + specular - 2.0 * a;\n    float c = a - specular;\n    float D = max(b * b - 4.0 * a * c, 0.0);\n    return clamp((-b + sqrt(D)) / (2.0 * a), 0.0, 1.0);\n  }\n\n  void getBaseColorAndMetallicRoughnessFromSpecularGlossiness(inout PBRData data) {\n    vec4 specularGlossiness = getSpecularGlossiness(data);\n\n    vec3 specular = specularGlossiness.rgb;\n    data.f0 = specular;\n\n    float glossiness = specularGlossiness.a;\n    data.roughness = 1.0 - glossiness;\n\n    vec4 diffuseRGBA = getDiffuse(data);\n    vec3 diffuse = diffuseRGBA.rgb;\n    data.opacity = diffuseRGBA.a;\n    float epsilon = 1e-6;\n    float a = 0.04;\n\n    // ported from https://github.com/KhronosGroup/glTF/blob/master/extensions/Khronos/KHR_materials_pbrSpecularGlossiness/examples/convert-between-workflows/js/three.pbrUtilities.js\n    float oneMinusSpecularStrength = 1.0 - maxComponent(specular);\n    data.metallic = solveMetallic(perceivedBrightness(diffuse), perceivedBrightness(specular), oneMinusSpecularStrength);\n\n    vec3 baseColorFromDiffuse = diffuse * oneMinusSpecularStrength / (1.0 - a) / max(1.0 - data.metallic, epsilon);\n    vec3 baseColorFromSpecular = (specular - a * (1.0 - data.metallic)) * (1.0 / max(data.metallic, epsilon));\n    data.baseColor = mix(baseColorFromDiffuse, baseColorFromSpecular, data.metallic * data.metallic);\n\n    #if defined(USE_VERTEX_COLORS) || defined(USE_INSTANCED_COLOR)\n      vec3 tint = decode(vColor, SRGB).rgb;\n      data.baseColor *= tint;\n      data.f0 *= tint;\n      data.opacity *= vColor.a;\n    #endif\n  }\n#endif\n";
export default _default;
